#define F_CPU   12000000L

#include <avr/io.h>
#include <avr/delay.h>
#include <avr/pgmspace.h>
#include <string.h>
#include "usbconfig.h"
#include "ataristyle.h"

static void atariStyleInit(void);
static void atariStyleUpdate(void);
static char atariStyleChanged(void);
static void atariStyleBuildReport(unsigned char *reportBuffer);

/* 
 * Bit 0: Up
 * Bit 1: Down
 * Bit 2: Left
 * Bit 3: Right
 * Bit 4: Button 1
 * Bit 5: Button 2
 * */
static unsigned char last_update_state=0;
static unsigned char last_reported_state=0;

/* Mapping:
 * 	Up.......: PC3
 * 	Down.....: PC2
 * 	Left.....: PC1
 * 	Right....: PC0
 * 	Button 1 : PB5
 * 	Button 2 : PB4
 */
static void atariStyleInit(void)
{
	// Directional button as inputs, pull up enabled.
	DDRC &= ~(0x0f);
	PORTC |= 0x0f;
	
	// Buttons as inputs, pull up enables.
	DDRB &= ~( (1<<PB5) | (1<<PB4) );
	PORTB |= (1<<PB5) | (1<<PB4);
}

static void atariStyleUpdate(void)
{
	last_update_state = PINC & 0xf;
	last_update_state |= (PINB) & ((1<<PB5) | (1<<PB4));
}

static char atariStyleChanged(void)
{
	return (last_update_state != last_reported_state);
}

#define REPORT_SIZE 3

static void atariStyleBuildReport(unsigned char *reportBuffer)
{
	int x,y;
	unsigned char tmp;
	
	if (reportBuffer)
	{
		y = x = 0x80;

		tmp = last_update_state ^ 0xff;
		
		if (tmp&0x01) { x = 0xff; }
		if (tmp&0x02) { x = 0x00; }
		if (tmp&0x04) { y = 0xff; }
		if (tmp&0x08) { y = 0x00; }

		reportBuffer[0] = x;
		reportBuffer[1] = y;
		reportBuffer[2] = 0;
		if (tmp&0x10) reportBuffer[2] |= 0x80;
		if (tmp&0x20) reportBuffer[2] |= 0x40;
	}
	last_reported_state = last_update_state;
}

const char atariStyle_usbHidReportDescriptor[] PROGMEM = {
	0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
	0x09, 0x05,                    // USAGE (Game Pad)
	0xa1, 0x01,                    // COLLECTION (Application)
	0x09, 0x01,                    //   USAGE (Pointer)
	0xa1, 0x00,                    //   COLLECTION (Physical)
	0x09, 0x30,                    //     USAGE (X)
	0x09, 0x31,                    //     USAGE (Y)
	0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
	0x26, 0xff, 0x00,              //     LOGICAL_MAXIMUM (255)
	0x75, 0x08,                    //     REPORT_SIZE (8)
	0x95, 0x02,                    //     REPORT_COUNT (2)
	0x81, 0x02,                    //     INPUT (Data,Var,Abs)
	0xc0,                          //   END_COLLECTION
	
	0x95, 0x06,                    //   REPORT_COUNT (6)
	0x75, 0x01,                    //   REPORT_SIZE (1)
	0x81, 0x03,                    //   INPUT (Const,Var,Abs)

	0x05, 0x09,                    //   USAGE_PAGE (Button)
	0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
	0x29, 0x02,                    //   USAGE_MAXIMUM (Button 2)
	0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
	0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
	0x75, 0x01,                    //   REPORT_SIZE (1)
	0x95, 0x02,                    //   REPORT_COUNT (2)
	0x81, 0x02,                    //   INPUT (Data,Var,Abs)

		
	0xc0                           // END_COLLECTION (application)
};

#define USBDESCR_DEVICE         1

// This is the same descriptor as in devdesc.c, but the product id is 0x0a9b 

const char atariStyle_usbDescrDevice[] PROGMEM = {    /* USB device descriptor */
    18,         /* sizeof(usbDescrDevice): length of descriptor in bytes */
    USBDESCR_DEVICE,    /* descriptor type */
    0x01, 0x01, /* USB version supported */
    USB_CFG_DEVICE_CLASS,
    USB_CFG_DEVICE_SUBCLASS,
    0,          /* protocol */
    8,          /* max packet size */
    USB_CFG_VENDOR_ID,  /* 2 bytes */
    0x9b, 0x0a,  /* 2 bytes */
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
#if USB_CFG_VENDOR_NAME_LEN
    1,          /* manufacturer string index */
#else
    0,          /* manufacturer string index */
#endif
#if USB_CFG_DEVICE_NAME_LEN
    2,          /* product string index */
#else
    0,          /* product string index */
#endif
#if USB_CFG_SERIAL_NUMBER_LENGTH
    3,          /* serial number string index */
#else
    0,          /* serial number string index */
#endif
    1,          /* number of configurations */
};

Gamepad atariStyleJoy = {
	report_size:	REPORT_SIZE,
	reportDescriptorSize: sizeof(atariStyle_usbHidReportDescriptor),
	deviceDescriptorSize: sizeof(atariStyle_usbDescrDevice),
	init: atariStyleInit,
	update: atariStyleUpdate,
	changed: atariStyleChanged,
	buildReport: atariStyleBuildReport
};

Gamepad *atariStyleGetGamepad(void)
{
	atariStyleJoy.reportDescriptor = (void*)atariStyle_usbHidReportDescriptor;
	atariStyleJoy.deviceDescriptor = (void*)atariStyle_usbDescrDevice;

	return &atariStyleJoy;
}


